#ifndef DUNE_CURVEDSURFACEGRID_CAPABILITIES_HH
#define DUNE_CURVEDSURFACEGRID_CAPABILITIES_HH

#include <cassert>
#include <utility>

#include <dune/common/hybridutilities.hh>
#include <dune/curvedsurfacegrid/declaration.hh>
#include <dune/curvedsurfacegrid/gridfunctions/gridfunction.hh>
#include <dune/grid/common/capabilities.hh>
#include <dune/grid/geometrygrid/capabilities.hh>

// Capabilities
// ------------

namespace Dune {
namespace Capabilities {

// Capabilities from dune-grid
// ---------------------------

template <class GridFunction, int order>
struct hasSingleGeometryType<CurvedSurfaceGrid<GridFunction,order>>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = hasSingleGeometryType<HostGrid>::v;
  static const unsigned int topologyId = hasSingleGeometryType<HostGrid>::topologyId;
};

template <class GridFunction, int order, int codim>
struct hasEntity<CurvedSurfaceGrid<GridFunction,order>, codim>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = hasEntity<HostGrid,codim>::v;
};

template <class GridFunction, int order, int codim>
struct hasEntityIterator<CurvedSurfaceGrid<GridFunction,order>, codim>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = hasEntityIterator<HostGrid,codim>::v;
};

/// \brief Implements geometry only for codim=0 entity
template <class GridFunction, int order, int codim>
struct hasGeometry<CurvedSurfaceGrid<GridFunction,order>, codim>
{
  static const bool v = (codim == 0);
};

template <class GridFunction, int order, int codim>
struct canCommunicate<CurvedSurfaceGrid<GridFunction,order>, codim>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = canCommunicate<HostGrid, codim>::v && hasEntity<HostGrid, codim>::v;
};

/// \brief Conformity of the grid is not guaranteed since it depends on the GridFunction that
/// \brief must be continuous in that case.
template <class GridFunction, int order>
struct isLevelwiseConforming<CurvedSurfaceGrid<GridFunction,order>>
{
  static const bool v = false;
};

/// \brief Conformity of the grid is not guaranteed since it depends on the GridFunction that
/// \brief must be continuous in that case.
template <class GridFunction, int order>
struct isLeafwiseConforming<CurvedSurfaceGrid<GridFunction,order>>
{
  static const bool v = false;
};

/// \brief Implements only partial backup-restore facilities, since the gridfunction is not
/// \brief backuped automatically.
template <class GridFunction, int order>
struct hasBackupRestoreFacilities<CurvedSurfaceGrid<GridFunction,order>>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = hasBackupRestoreFacilities<HostGrid>::v;
};


// hasHostEntity
// -------------

template <class GridFunction, int order, int codim>
struct hasHostEntity<CurvedSurfaceGrid<GridFunction,order>, codim>
{
  using HostGrid = GridOf_t<GridFunction>;
  static const bool v = hasEntity<HostGrid, codim>::v;
};

} // end namespace Capabilities
} // end namespace Dune

#endif // DUNE_CURVEDSURFACEGRID_CAPABILITIES_HH
