#include "config.h"

#include <type_traits>

#include <dune/common/parallel/mpihelper.hh>
#include <dune/curvedsurfacegrid/curvedsurfacegrid.hh>
#include <dune/grid/io/file/gmshreader.hh>

using namespace Dune;

#if HAVE_DUNE_FOAMGRID
  #include <dune/foamgrid/foamgrid.hh>
  using GridType = FoamGrid<2,3>;
#elif HAVE_DUNE_ALUGRID
  #include <dune/alugrid/grid.hh>
  using GridType = Dune::ALUGrid<2,3,Dune::simplex,Dune::conforming>;
#endif

template <int order>
using Order_t = std::integral_constant<int,order>;

int main(int argc, char** argv)
{
  using namespace Dune;
  MPIHelper::instance(argc, argv);

  // Construct a reference grid
  auto refGrid = GmshReader<GridType>::read(DUNE_GRID_PATH "sphere.msh");

  // Define the geometry mapping
  auto sphere = [](const auto& x) { return x / x.two_norm(); };

  // Wrap the reference grid to build a curved grid
  CurvedSurfaceGrid grid{*refGrid, sphere, Order_t<3>{}};
}